@@ -1,7 +1,7 @@
 /* Program to write C++-suitable header files from a Java(TM) .class
    file.  This is similar to SUN's javah.
 
-Copyright (C) 1996, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005
+Copyright (C) 1996, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006
 Free Software Foundation, Inc.
 
 This file is part of GCC.
@@ -18,8 +18,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GCC; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  
+the Free Software Foundation, 51 Franklin Street, Fifth Floor,
+Boston, MA 02110-1301, USA.  
 
 Java and all Java-based marks are trademarks or registered trademarks
 of Sun Microsystems, Inc. in the United States and other countries.
@@ -145,7 +145,7 @@ static void print_method_info (FILE*, JCF*, int, int, JCF_u2);
 static void print_c_decl (FILE*, JCF*, int, int, int, const char *, int);
 static void print_stub_or_jni (FILE*, JCF*, int, int, int, const char *, int);
 static void print_full_cxx_name (FILE*, JCF*, int, int, int, const char *, int);
-static void decompile_method (FILE*, JCF*, int);
+static void decompile_method (FILE*, JCF*, int) ATTRIBUTE_UNUSED;
 static void add_class_decl (FILE*, JCF*, JCF_u2);
 
 static void print_name (FILE *, JCF *, int);
@@ -164,7 +164,7 @@ static const unsigned char *
   decode_signature_piece (FILE *, const unsigned char *,
 			  const unsigned char *, int *);
 static void print_class_decls (FILE *, JCF *, int);
-static void error (const char *gmsgid, ...);
+static void error (const char *gmsgid, ...) ATTRIBUTE_PRINTF_1;
 static void usage (void) ATTRIBUTE_NORETURN;
 static void help (void) ATTRIBUTE_NORETURN;
 static void version (void) ATTRIBUTE_NORETURN;
@@ -250,8 +250,14 @@ static int is_first_data_member = 0;
       }									\
   }
 
+/* Only include byte-code decompilation optimizations for ELF targets
+   since the generated headers are only known to work with ELF weak
+   symbol semantics. Specifically, these optimizations are known to
+   not work on PE-COFF and possibly others.  */
+#ifdef OBJECT_FORMAT_ELF
 #define HANDLE_CODE_ATTRIBUTE(MAX_STACK, MAX_LOCALS, CODE_LENGTH)	\
   if (out && method_declared) decompile_method (out, jcf, CODE_LENGTH);
+#endif
 
 static int decompiled = 0;
 #define HANDLE_END_METHOD()				\
@@ -594,7 +600,7 @@ cxx_keyword_subst (const unsigned char *str, int length)
 	     are `$'.  */
 	  if (i == length)
 	    {
-	      char *dup = xmalloc (2 + length - min_length + kwl);
+	      char *dup = XNEWVEC (char, 2 + length - min_length + kwl);
 	      strcpy (dup, cxx_keywords[mid]);
 	      for (i = kwl; i < length + 1; ++i)
 		dup[i] = '$';
@@ -713,7 +719,7 @@ get_field_name (JCF *jcf, int name_index, JCF_u2 flags)
 	  return NULL;
 	}
 
-      override = xmalloc (length + 3);
+      override = XNEWVEC (char, length + 3);
       memcpy (override, name, length);
       strcpy (override + length, "__");
     }
@@ -923,13 +929,13 @@ print_method_info (FILE *stream, JCF* jcf, int name_index, int sig_index,
     {
       struct method_name *nn;
 
-      nn = xmalloc (sizeof (struct method_name));
-      nn->name = xmalloc (length);
+      nn = XNEW (struct method_name);
+      nn->name = XNEWVEC (unsigned char, length);
       memcpy (nn->name, str, length);
       nn->length = length;
       nn->next = method_name_list;
       nn->sig_length = JPOOL_UTF_LENGTH (jcf, sig_index);
-      nn->signature = xmalloc (nn->sig_length);
+      nn->signature = XNEWVEC (unsigned char, nn->sig_length);
       nn->is_native = METHOD_IS_NATIVE (flags);
       memcpy (nn->signature, JPOOL_UTF_DATA (jcf, sig_index),
 	      nn->sig_length);
@@ -1234,7 +1240,7 @@ throwable_p (const unsigned char *clname)
 
   for (length = 0; clname[length] != ';' && clname[length] != '\0'; ++length)
     ;
-  current = ALLOC (length + 1);
+  current = XNEWVEC (unsigned char, length + 1);
   for (i = 0; i < length; ++i)
     current[i] = clname[i] == '/' ? '.' : clname[i];
   current[length] = '\0';
@@ -1270,7 +1276,7 @@ throwable_p (const unsigned char *clname)
       jcf_parse_class (&jcf);
 
       tmp = (unsigned char *) super_class_name (&jcf, &super_length);
-      super = ALLOC (super_length + 1);
+      super = XNEWVEC (unsigned char, super_length + 1);
       memcpy (super, tmp, super_length);      
       super[super_length] = '\0';
 
@@ -1830,8 +1836,8 @@ print_include (FILE *out, const unsigned char *utf8, int len)
 	return;
     }
 
-  incl = xmalloc (sizeof (struct include));
-  incl->name = xmalloc (len + 1);
+  incl = XNEW (struct include);
+  incl->name = XNEWVEC (char, len + 1);
   strncpy (incl->name, (const char *) utf8, len);
   incl->name[len] = '\0';
   incl->next = all_includes;
@@ -1916,8 +1922,8 @@ add_namelet (const unsigned char *name, const unsigned char *name_limit,
 
   if (n == NULL)
     {
-      n = xmalloc (sizeof (struct namelet));
-      n->name = xmalloc (p - name + 1);
+      n = XNEW (struct namelet);
+      n->name = XNEWVEC (char, p - name + 1);
       strncpy (n->name, (const char *) name, p - name);
       n->name[p - name] = '\0';
       n->is_class = (p == name_limit);
@@ -2167,7 +2173,7 @@ process_file (JCF *jcf, FILE *out)
 	  if (len > 6 && ! strcmp (&jcf->classname[len - 6], ".class"))
 	    len -= 6;
 	  /* Turn the class name into a file name.  */
-	  name = xmalloc (len + 1);
+	  name = XNEWVEC (char, len + 1);
 	  for (i = 0; i < len; ++i)
 	    name[i] = jcf->classname[i] == '.' ? '/' : jcf->classname[i];
 	  name[i] = '\0';
@@ -2397,7 +2403,7 @@ static void
 version (void)
 {
   printf (TOOLNAME " (GCC) %s\n\n", version_string);
-  printf ("Copyright %s 2004 Free Software Foundation, Inc.\n", _("(C)"));
+  printf ("Copyright %s 2006 Free Software Foundation, Inc.\n", _("(C)"));
   printf (_("This is free software; see the source for copying conditions.  There is NO\n"
 	    "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\n"));
   exit (0);
@@ -2411,6 +2417,7 @@ main (int argc, char** argv)
   char *output_file = NULL;
   int emit_dependencies = 0, suppress_output = 0;
   int opt;
+  int local_found_error;
 
   /* Unlock the stdio streams.  */
   unlock_std_streams ();
@@ -2477,25 +2484,25 @@ main (int argc, char** argv)
 
 	case OPT_PREPEND:
 	  if (prepend_count == 0)
-	    prepend_specs = ALLOC (argc * sizeof (char*));
+	    prepend_specs = XNEWVEC (char *, argc);
 	  prepend_specs[prepend_count++] = optarg;
 	  break;
 
 	case OPT_FRIEND:
 	  if (friend_count == 0)
-	    friend_specs = ALLOC (argc * sizeof (char*));
+	    friend_specs = XNEWVEC (char *, argc);
 	  friend_specs[friend_count++] = optarg;
 	  break;
 
 	case OPT_ADD:
 	  if (add_count == 0)
-	    add_specs = ALLOC (argc * sizeof (char*));
+	    add_specs = XNEWVEC (char *, argc);
 	  add_specs[add_count++] = optarg;
 	  break;
 
 	case OPT_APPEND:
 	  if (append_count == 0)
-	    append_specs = ALLOC (argc * sizeof (char*));
+	    append_specs = XNEWVEC (char *, argc);
 	  append_specs[append_count++] = optarg;
 	  break;
 
@@ -2558,12 +2565,18 @@ main (int argc, char** argv)
       exit (1);
     }
 
+  local_found_error = 0;
   for (argi = optind; argi < argc; argi++)
     {
       char *classname = argv[argi];
-      char *current_output_file;
+      char *current_output_file = NULL;
       const char *classfile_name;
 
+      /* We reset the error state here so that we can detect errors
+	 that occur when processing this file, so the output can be
+	 unlinked if need be.  */
+      found_error = 0;
+
       if (verbose)
 	printf (_("Processing %s\n"), classname);
       if (! output_file)
@@ -2595,7 +2608,7 @@ main (int argc, char** argv)
 	{
 	  int dir_len = strlen (output_directory);
 	  int i, classname_length = strlen (classname);
-	  current_output_file = ALLOC (dir_len + classname_length + 5);
+	  current_output_file = XNEWVEC (char, dir_len + classname_length + 5);
 	  strcpy (current_output_file, output_directory);
 	  if (dir_len > 0 && output_directory[dir_len-1] != '/')
 	    current_output_file[dir_len++] = '/';
@@ -2639,13 +2652,22 @@ main (int argc, char** argv)
       free_method_name_list ();
       process_file (&jcf, out);
       JCF_FINISH (&jcf);
+
+      /* If we found an error and we're writing to a real file,
+	 delete it.  */
+      if (found_error && ! suppress_output && current_output_file != NULL
+	  && strcmp (current_output_file, "-"))
+	unlink (current_output_file);
+
       if (current_output_file != output_file)
 	free (current_output_file);
       jcf_dependency_write ();
+
+      local_found_error |= found_error;
     }
 
   if (out != NULL && out != stdout)
     fclose (out);
 
-  return found_error;
+  return local_found_error;
 }
